;;; -*- Mode: Lisp; Package: PVS -*-
(in-package "PVS")  ;; creates package for abstract syntax. 

(in-package "PVS")  ;; enters package for generated code.  

(use-package '("NEWATTR" "LANG" "SB-RUNTIME" "SORT" "TERM" "OCC" "OPER" "ERGOLISP"))


(export '())
#+(and allegro-version>= (version>= 8 2))(eval-when (:execute :compile-toplevel :load-toplevel)(setq *readtable* cl::*pvs-readtable*))
(defparameter *PVS-OPSIG-TABLE*
    (make-opsig-table
     '((#^set-formals
        . #@("pvs" :=
             (#t(:sort set-formal) #t(:sort lambda-formals-opt-0)
              #t(:sort set-formals))
             :-> #t(:sort set-formals)))
       (#^rec-subtype-of
        . #@("pvs" := nil :-> #t(:sort judgement-opt-0)))
       (#^datatype
        . #@("pvs" := (#t(:sort adtbody) #t(:sort pidop)) :->
             #t(:sort datatype)))
       (#^fieldappl
        . #@("pvs" := (#t(:sort expr) #t(:sort sb-runtime::id)) :->
             #t(:sort expr)))
       (#^challenge . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^eq . #@("pvs" := nil :-> #t(:sort lib-decl-opt-0)))
       (#^exists . #@("pvs" := nil :-> #t(:sort exists-op)))
       (#^fieldex
        . #@("pvs" := (#t(:sort sb-runtime::id) #t(:sort expr-opt-0))
             :-> #t(:sort expr)))
       (#^no-theory-formals
        . #@("pvs" := nil :-> #t(:sort adt-or-theory-opt-0)))
       (#^obligation . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^idop
        . #@("pvs" := (#t(:sort opsym)) :-> #t(:sort idop-not-number)))
       (#^noactuals . #@("pvs" := nil :-> #t(:sort expr-opt-0)))
       (#^postulate . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^nopred . #@("pvs" := nil :-> #t(:sort name-expr-opt-0)))
       (#^nomap . #@("pvs" := nil :-> #t(:sort modname-opt-1)))
       (#^⊎ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^implies . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^table-expr
        . #@("pvs" :=
             (#t(:sort table-expr-opt-0) #t(:sort table-expr-opt-1)
              #t(:sort table-expr-opt-2) #t(:sort table-entries))
             :-> #t(:sort table-expr)))
       (#^adt-or-modules
        . #@("pvs" := (#t(:list (:sort adt-or-theory))) :->
             #t(:sort adt-or-theories)))
       (#^datatypes
        . #@("pvs" :=
             (#t(:sort pidops) #t(:sort adtbody) #t(:sort pidop)) :->
             #t(:sort datatype)))
       (#^pidacts
        . #@("pvs" := (#t(:sort pidop) #t(:sort actuals)) :->
             #t(:sort name-opt-0)))
       (#^± . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^← . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^true . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊏ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^>>= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^nonempty-type . #@("pvs" := nil :-> #t(:sort type-keyword)))
       (#^table-entry
        . #@("pvs" :=
             (#t(:list
                 (:union (:op #^else) (:sort expr)
                  (:op #^notableentry))))
             :-> #t(:sort table-entry)))
       (#^bracket-expr
        . #@("pvs" := (#t(:sort ⌞⌟) #t(:sort btup)) :->
             #t(:sort bracket-expr)))
       (#^// . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^notgt . #@("pvs" := nil :-> #t(:sort modname-opt-2)))
       (#^extended-type
        . #@("pvs" :=
             (#t(:sort type-expr-sans-name) #t(:sort type-expr)) :->
             #t(:sort type-ex)))
       (#^↑ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊐ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^typed-ids
        . #@("pvs" :=
             (#t(:sort idops) #t(:sort set-expr-opt-0)
              #t(:sort typed-id-opt-0))
             :-> #t(:sort typed-ids)))
       (#^⌊⌋ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^conversionminus
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort conversion)))
       (#^|<:| . #@("pvs" := nil :-> #t(:sort type-def-alt-0)))
       (#^axiom . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^→ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊑ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^xor . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^judgement-elt
        . #@("pvs" := (#t(:sort judgement) #t(:sort assuming-opt-0))
             :-> #t(:sort theory-elt)))
       (#^<> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^and . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^no-offset . #@("pvs" := nil :-> #t(:sort units-decl-opt-0)))
       (#^comp-type-expr-NULL-2
        . #@("pvs" := (#t(:null)) :-> #t(:sort comp-type-expr-opt-2)))
       (#^|##| . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^assumption
        . #@("pvs" := (#t(:sort expr)) :-> #t(:sort assumption)))
       (#^bind-expr
        . #@("pvs" := (#t(:sort expr-alt-0) #t(:sort lambda-body)) :->
             #t(:sort expr)))
       (#^↓ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^array . #@("pvs" := nil :-> #t(:sort fun-array)))
       (#^lambda-body
        . #@("pvs" :=
             (#t(:sort lambda-formals) #t(:sort set-expr-opt-0)
              #t(:sort expr))
             :-> #t(:sort lambda-body)))
       (#^assign-skoname
        . #@("pvs" := (#t(:sort sb-runtime::id) #t(:sort number)) :->
             #t(:sort assign-arg)))
       (#^⊒ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noass . #@("pvs" := nil :-> #t(:sort theory-opt-1)))
       (#^=> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^expname
        . #@("pvs" := (#t(:sort pidop) #t(:sort expname-opt-0)) :->
             #t(:sort expname)))
       (#^rewrite-name
        . #@("pvs" :=
             (#t(:sort name) #t(:sort rewrite-name-or-fnum-opt-0)
              #t(:sort rewrite-name-opt-0))
             :-> #t(:sort rewrite-name)))
       (#^∓ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^typed-id
        . #@("pvs" :=
             (#t(:sort idop) #t(:sort set-expr-opt-0)
              #t(:sort typed-id-opt-0))
             :-> #t(:sort typed-id)))
       (#^comma . #@("pvs" := nil :-> #t(:sort lambda-formals-opt-0)))
       (#^units-decl
        . #@("pvs" := (#t(:sort units-expr) #t(:sort units-decl-opt-0))
             :-> #t(:sort units-decl)))
       (#^<= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊓ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^id-assign
        . #@("pvs" := (#t(:sort sb-runtime::id)) :->
             #t(:sort assign-arg)))
       (#^>> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^string-expr
        . #@("pvs" := (#t(:sort string)) :-> #t(:sort expr)))
       (#^def-decl
        . #@("pvs" :=
             (#t(:sort type-expr) #t(:sort expr) #t(:sort expr)
              #t(:sort def-decl-opt-0))
             :-> #t(:sort def-decl)))
       (#^const-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort const-value)) :->
             #t(:sort const-decl)))
       (#^∔ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^rec-has-type
        . #@("pvs" := nil :-> #t(:sort judgement-opt-0)))
       (#^no-sec . #@("pvs" := nil :-> #t(:sort assuming-opt-1)))
       (#^cond-case
        . #@("pvs" := (#t(:sort expr) #t(:sort expr)) :->
             #t(:sort cond-case)))
       (#^== . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊔ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^>= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≠ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^enumtype
        . #@("pvs" := (#t(:sort set-formals)) :->
             #t(:sort type-expr-sans-name)))
       (#^⊕ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^or . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^false . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^notype . #@("pvs" := nil :-> #t(:sort ename-opt-0)))
       (#^ftheory-decl
        . #@("pvs" := (#t(:sort theory-decl-modname)) :->
             #t(:sort theory-formal-decl)))
       (#^codatatypes
        . #@("pvs" :=
             (#t(:sort pidops) #t(:sort adtbody) #t(:sort pidop)) :->
             #t(:sort datatype)))
       (#^⋀ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^btup
        . #@("pvs" := (#t(:list (:sort expr))) :-> #t(:sort btup)))
       (#^≡ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≁ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊖ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^funtype
        . #@("pvs" :=
             (#t(:sort comp-type-expr-opt-0)
              #t(:sort comp-type-expr-opt-1)
              #t(:sort comp-type-expr-opt-2))
             :-> #t(:sort comp-type-expr)))
       (#^names
        . #@("pvs" := (#t(:list (:sort name))) :-> #t(:sort names)))
       (#^arr . #@("pvs" := nil :-> #t(:sort assignment-alt-0)))
       (#^⋁ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^uninterp-const-decl
        . #@("pvs" := (#t(:sort type-expr)) :-> #t(:sort const-decl)))
       (#^⊗ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^type . #@("pvs" := nil :-> #t(:sort mapping-lhs-opt-0)))
       (#^selection-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort selection-opt-0)))
       (#^∘ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^update-expr
        . #@("pvs" := (#t(:sort expr) #t(:sort assignments)) :->
             #t(:sort expr)))
       (#^⋂ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noexp . #@("pvs" := nil :-> #t(:sort theory-opt-0)))
       (#^⌞⌟ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^cases-expr
        . #@("pvs" :=
             (#t(:sort expr) #t(:sort args) #t(:sort expr-opt-1)) :->
             #t(:sort expr)))
       (#^conversion
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort conversion)))
       (#^pdf
        . #@("pvs" := (#t(:list (:sort pdformals))) :->
             #t(:sort assuming-opt-2)))
       (#^≃ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊘ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^|;| . #@("pvs" := nil :-> #t(:sort semic)))
       (#^⋃ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^expnames
        . #@("pvs" := (#t(:list (:sort expname))) :->
             #t(:sort exportings)))
       (#^extended-type-appl
        . #@("pvs" :=
             (#t(:sort name) #t(:sort arguments) #t(:sort type-expr))
             :-> #t(:sort type-ex)))
       (#^assignments
        . #@("pvs" := (#t(:list (:sort assignment))) :->
             #t(:sort assignments)))
       (#^atdecl . #@("pvs" := nil :-> #t(:sort assuming-opt-3)))
       (#^assuming
        . #@("pvs" :=
             (#t(:sort assuming-opt-1) #t(:sort pidops)
              #t(:sort adt-or-theory-opt-0) #t(:sort assuming-opt-2)
              #t(:sort assuming-decl) #t(:sort assuming-opt-0)
              #t(:sort assuming-opt-3))
             :-> #t(:sort assuming)))
       (#^≤ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊙ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^< . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^corollary . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^noden . #@("pvs" := nil :-> #t(:sort rational-opt-1)))
       (#^noexpkind . #@("pvs" := nil :-> #t(:sort expname-opt-0)))
       (#^conversionplus
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort conversion)))
       (#^typeplus . #@("pvs" := nil :-> #t(:sort type-keyword)))
       (#^emptytuple
        . #@("pvs" := nil :-> #t(:sort comp-type-expr-opt-1)))
       (#^√ . #@("pvs" := nil :-> #t(:sort √)))
       (#^<<= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^idopappl
        . #@("pvs" := (#t(:sort idop) #t(:sort deptypes)) :->
             #t(:sort idopappl)))
       (#^rewrite-fnum
        . #@("pvs" :=
             (#t(:sort fnum) #t(:sort rewrite-name-or-fnum-opt-0)) :->
             #t(:sort rewrite-name-or-fnum)))
       (#^expr-as-type
        . #@("pvs" := (#t(:sort expr)) :->
             #t(:sort type-expr-sans-name)))
       (#^≅ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≥ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^list-expr
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort bracket-expr)))
       (#^= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^◁ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⌈⌉ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^▷ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^typed
        . #@("pvs" := (#t(:sort type-expr)) :->
             #t(:sort mapping-lhs-opt-0)))
       (#^mapping-rename
        . #@("pvs" := (#t(:sort mapping-lhs) #t(:sort mapping-rhs)) :->
             #t(:sort theory-decl-mapping)))
       (#^lambda . #@("pvs" := nil :-> #t(:sort lambda-op)))
       (#^set-expr
        . #@("pvs" :=
             (#t(:sort set-formals) #t(:sort set-expr-opt-0)
              #t(:sort expr))
             :-> #t(:sort set-expr)))
       (#^> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≦ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^codatatype
        . #@("pvs" := (#t(:sort adtbody) #t(:sort pidop)) :->
             #t(:sort datatype)))
       (#^⊛ . #@("pvs" := nil :-> #t(:sort ⊛)))
       (#^skovar
        . #@("pvs" := (#t(:sort sb-runtime::id) #t(:sort number)) :->
             #t(:sort expr)))
       (#^adtcase-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort adtcase-opt-0)))
       (#^macro
        . #@("pvs" := nil :-> #t(:sort rewrite-name-or-fnum-opt-0)))
       (#^∼ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^neg . #@("pvs" := nil :-> #t(:sort rational-opt-0)))
       (#^sec . #@("pvs" := nil :-> #t(:sort assuming-opt-1)))
       (#^↝ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≧ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≇ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^λ . #@("pvs" := nil :-> #t(:sort lambda-op)))
       (#^elsif
        . #@("pvs" := (#t(:sort expr) #t(:sort expr)) :->
             #t(:sort elsif)))
       (#^actuals
        . #@("pvs" := (#t(:list (:sort actual))) :->
             #t(:sort actuals1)))
       (#^theory-formal-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort theory-formal-opt-0)))
       (#^≨ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≈ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^nocolvar . #@("pvs" := nil :-> #t(:sort table-expr-opt-1)))
       (#^type-coercion
        . #@("pvs" := (#t(:sort type-expr)) :->
             #t(:sort type-coercion)))
       (#^if . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^notableentry
        . #@("pvs" := nil :-> #t(:sort table-entry-opt-0)))
       (#^modname
        . #@("pvs" :=
             (#t(:sort sb-runtime::id) #t(:sort modname-opt-0)
              #t(:sort expr-opt-0) #t(:sort theory-decl-modname-opt-0)
              #t(:sort theory-decl-modname-opt-1))
             :-> #t(:sort theory-decl-modname)))
       (#^expr-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort expr-opt-1)))
       (#^⋈ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^type-name
        . #@("pvs" := (#t(:sort name)) :-> #t(:sort type-ex)))
       (#^type-decl
        . #@("pvs" :=
             (#t(:sort type-keyword) #t(:sort type-def)
              #t(:sort type-decl-opt-0))
             :-> #t(:sort type-decl)))
       (#^projex
        . #@("pvs" := (#t(:sort number) #t(:sort expr-opt-0)) :->
             #t(:sort expr)))
       (#^≩ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≉ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊞ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^elsifs
        . #@("pvs" := (#t(:list (:op #^elsif))) :-> #t(:sort elsifs)))
       (#^mappings
        . #@("pvs" := (#t(:list (:sort theory-decl-mapping))) :->
             #t(:sort theory-decl-mappings)))
       (#^binds
        . #@("pvs" := (#t(:list (:sort simplebind))) :->
             #t(:sort bind-alt-0)))
       (#^not . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^lib-fdecl
        . #@("pvs" := (#t(:sort idop) #t(:sort lib-decl)) :->
             #t(:sort theory-formal-opt-1)))
       (#^≪ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊟ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^<=> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^theorem . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^auto-rewriteplus
        . #@("pvs" := (#t(:list (:sort rewrite-name))) :->
             #t(:sort auto-rewrite)))
       (#^args
        . #@("pvs" := (#t(:list (:sort assign-arg))) :->
             #t(:sort args)))
       (#^@@ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^/\\ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≫ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^◇ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^lib-decl
        . #@("pvs" := (#t(:sort lib-decl-opt-0) #t(:sort string)) :->
             #t(:sort lib-decl)))
       (#^ceq . #@("pvs" := nil :-> #t(:sort assignment-alt-0)))
       (#^assign-tuple
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort assign-arg)))
       (#^formula-decl
        . #@("pvs" := (#t(:sort formula-name) #t(:sort expr)) :->
             #t(:sort formula-decl)))
       (#^mapping-lhs
        . #@("pvs" :=
             (#t(:sort pidop) #t(:sort adt-or-theory-opt-0)
              #t(:sort assuming-opt-2) #t(:sort mapping-lhs-opt-0))
             :-> #t(:sort mapping-lhs)))
       (#^o . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^adtdecl
        . #@("pvs" := (#t(:sort pidops) #t(:sort type-expr)) :->
             #t(:sort adtdecl)))
       (#^comp-type-expr-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort comp-type-expr-opt-0)))
       (#^let-bindings
        . #@("pvs" := (#t(:list (:sort bind))) :->
             #t(:sort let-bindings)))
       (#^≍ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^ename
        . #@("pvs" := (#t(:sort name) #t(:sort ename-opt-0)) :->
             #t(:sort ename)))
       (#^bname
        . #@("pvs" := (#t(:sort name) #t(:sort number)) :->
             #t(:sort bname)))
       (#^importing-elt
        . #@("pvs" := (#t(:sort importing) #t(:sort assuming-opt-0))
             :-> #t(:sort importing-elt)))
       (#^sublemma . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^jdecl-expr
        . #@("pvs" := (#t(:sort expr)) :-> #t(:sort jdecl)))
       (#^col-heading
        . #@("pvs" := (#t(:sort expr) #t(:sort colrest)) :->
             #t(:sort col-heading)))
       (#^conversion-elt
        . #@("pvs" := (#t(:sort conversion) #t(:sort assuming-opt-0))
             :-> #t(:sort theory-elt)))
       (#^projappl
        . #@("pvs" := (#t(:sort expr) #t(:sort number)) :->
             #t(:sort expr)))
       (#^coind-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort expr)) :->
             #t(:sort coind-decl)))
       (#^≮ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≎ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^pidops
        . #@("pvs" := (#t(:list (:sort pidop))) :-> #t(:sort pidops)))
       (#^rational
        . #@("pvs" :=
             (#t(:sort rational-opt-0) #t(:sort number)
              #t(:sort rational-opt-1))
             :-> #t(:sort rational)))
       (#^adtcase
        . #@("pvs" := (#t(:sort adtcase-opt-0) #t(:sort pidop)) :->
             #t(:sort adtcase)))
       (#^where-expr
        . #@("pvs" := (#t(:sort let-bindings) #t(:sort expr)) :->
             #t(:sort expr)))
       (#^modnames
        . #@("pvs" := (#t(:list (:sort modname))) :->
             #t(:sort modnames)))
       (#^set-id
        . #@("pvs" := (#t(:sort idop) #t(:sort ename-opt-0)) :->
             #t(:sort set-formal)))
       (#^≯ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≏ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^ghost
        . #@("pvs" := (#t(:sort type-ex)) :-> #t(:sort type-expr)))
       (#^forall . #@("pvs" := nil :-> #t(:sort forall-op)))
       (#^equal . #@("pvs" := nil :-> #t(:sort type-def-alt-0)))
       (#^⇐ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^idopappls
        . #@("pvs" := (#t(:list (:sort idopappl))) :->
             #t(:sort idopappls)))
       (#^uninterp-type-decl
        . #@("pvs" := (#t(:sort type-keyword)) :->
             #t(:sort type-decl)))
       (#^importing
        . #@("pvs" := (#t(:list (:sort importing-item))) :->
             #t(:sort importing)))
       (#^name-bind-expr
        . #@("pvs" := (#t(:sort sb-runtime::id) #t(:sort lambda-body))
             :-> #t(:sort expr)))
       (#^id-adtcase
        . #@("pvs" := (#t(:sort pidop) #t(:sort adtcase)) :->
             #t(:sort id-datatype-elt)))
       (#^⟦⟧ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^expformula . #@("pvs" := nil :-> #t(:sort expname-opt-0)))
       (#^\|- . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≰ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^≐ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^let-expr
        . #@("pvs" := (#t(:sort let-bindings) #t(:sort expr)) :->
             #t(:sort expr)))
       (#^noformals . #@("pvs" := nil :-> #t(:sort assuming-opt-2)))
       (#^type-def
        . #@("pvs" := (#t(:sort type-def-alt-0) #t(:sort type-expr))
             :-> #t(:sort type-def)))
       (#^andthen . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⇑ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^lambda-formals
        . #@("pvs" :=
             (#t(:sort lambda-formal) #t(:sort lambda-formals-opt-0)
              #t(:sort lambda-formals))
             :-> #t(:sort lambda-formals)))
       (#^conjecture . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^table-entries
        . #@("pvs" := (#t(:list (:sort table-entry))) :->
             #t(:sort table-entries)))
       (#^|(\|\|)| . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^cond-expr
        . #@("pvs" := (#t(:sort args) #t(:sort expr-opt-2)) :->
             #t(:sort expr)))
       (#^≱ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^tuple-expr
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort tuple-expr)))
       (#^⇒ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noexportingmods
        . #@("pvs" := nil :-> #t(:sort exporting-opt-0)))
       (#^extended-type-name
        . #@("pvs" := (#t(:sort name) #t(:sort type-expr)) :->
             #t(:sort type-ex)))
       (#^nocolheading
        . #@("pvs" := nil :-> #t(:sort table-expr-opt-2)))
       (#^theory-decl
        . #@("pvs" := (#t(:sort theory-decl-modname)) :->
             #t(:sort theory-decl)))
       (#^\|> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^expmodall . #@("pvs" := nil :-> #t(:sort exportingmods)))
       (#^corec-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort expr)) :->
             #t(:sort corec-decl)))
       (#^^^ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^application
        . #@("pvs" := (#t(:sort expr) #t(:sort fun-arguments)) :->
             #t(:sort expr)))
       (#^⇓ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^expbuts
        . #@("pvs" := (#t(:list (:sort expname))) :->
             #t(:sort exportings-opt-0)))
       (#^actuals2
        . #@("pvs" := (#t(:sort actuals1) #t(:sort actuals1)) :->
             #t(:sort actuals)))
       (#^adtdecls
        . #@("pvs" := (#t(:list (:op #^adtdecl))) :->
             #t(:sort constructor)))
       (#^judgement
        . #@("pvs" :=
             (#t(:sort jdecls) #t(:sort judgement-opt-0)
              #t(:sort type-expr))
             :-> #t(:sort judgement)))
       (#^theory-part
        . #@("pvs" := (#t(:list (:sort theory-elt))) :->
             #t(:sort theory-part)))
       (#^nocomma
        . #@("pvs" := nil :-> #t(:sort lambda-formals-opt-0)))
       (#^name
        . #@("pvs" :=
             (#t(:sort idop) #t(:sort modname-opt-0)
              #t(:sort expr-opt-0) #t(:sort name-opt-0)
              #t(:sort modname-opt-1) #t(:sort modname-opt-2))
             :-> #t(:sort name)))
       (#^◯ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^proposition . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^unique-name
        . #@("pvs" := (#t(:sort name) #t(:sort unique-name-opt-0)) :->
             #t(:sort unique-name)))
       (#^nomod . #@("pvs" := nil :-> #t(:sort name-opt-0)))
       (#^units-term
        . #@("pvs" :=
             (#t(:sort /) #t(:sort units-expr) #t(:sort units-expr))
             :-> #t(:sort units-expr)))
       (#^jdecl-type
        . #@("pvs" := (#t(:sort comp-type-expr)) :-> #t(:sort jdecl)))
       (#^⇔ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^\|= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^norowvar . #@("pvs" := nil :-> #t(:sort table-expr-opt-0)))
       (#^from . #@("pvs" := nil :-> #t(:sort type-def-alt-0)))
       (#^adt-or-module
        . #@("pvs" :=
             (#t(:sort sb-runtime::id) #t(:sort adt-or-theory-opt-0)
              #t(:sort adt-or-theory-alt-0))
             :-> #t(:sort adt-or-theory)))
       (#^subtype
        . #@("pvs" := (#t(:sort set-formals) #t(:sort expr)) :->
             #t(:sort type-expr-sans-name)))
       (#^struct-subtype
        . #@("pvs" := (#t(:sort type-expr)) :->
             #t(:sort theory-formal-decl-opt-0)))
       (#^assuming-part
        . #@("pvs" := (#t(:list (:sort assuming))) :->
             #t(:sort assumings)))
       (#^module
        . #@("pvs" :=
             (#t(:sort theory-opt-0) #t(:sort theory-opt-1)
              #t(:sort theory-opt-2) #t(:sort sb-runtime::id))
             :-> #t(:sort theory)))
       (#^when . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noexpr . #@("pvs" := nil :-> #t(:sort def-decl-opt-0)))
       (#^• . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^ind-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort expr)) :->
             #t(:sort ind-decl)))
       (#^⟪⟫ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^assign-num
        . #@("pvs" := (#t(:sort number)) :-> #t(:sort assign-arg)))
       (#^theory-formal
        . #@("pvs" :=
             (#t(:sort theory-formal-opt-0) #t(:sort idopappls)
              #t(:sort theory-formal-decl))
             :-> #t(:sort theory-formal)))
       (#^∀ . #@("pvs" := nil :-> #t(:sort forall-op)))
       (#^pos . #@("pvs" := nil :-> #t(:sort rational-opt-0)))
       (#^÷ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^× . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^recordtype
        . #@("pvs" := (#t(:sort fields)) :-> #t(:sort recordtype)))
       (#^coercion
        . #@("pvs" := (#t(:sort expr) #t(:sort type-coercion)) :->
             #t(:sort expr)))
       (#^auto-rewrite-elt
        . #@("pvs" := (#t(:sort auto-rewrite) #t(:sort assuming-opt-0))
             :-> #t(:sort theory-elt)))
       (#^⊠ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^|#| . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^〈〉 . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^{\|\|} . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^[\|\|] . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^cotupletype
        . #@("pvs" := (#t(:list (:sort dep-type-expr))) :->
             #t(:sort dep-type-exprs)))
       (#^fconst-decl
        . #@("pvs" := (#t(:sort type-expr)) :->
             #t(:sort theory-formal-decl)))
       (#^no-type-expr . #@("pvs" := nil :-> #t(:sort set-expr-opt-0)))
       (#^proj-assign
        . #@("pvs" := (#t(:sort number)) :-> #t(:sort assign-arg)))
       (#^≗ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^deptypes
        . #@("pvs" := (#t(:list (:sort type-expr))) :->
             #t(:sort deptypes)))
       (#^var-decl
        . #@("pvs" := (#t(:sort type-expr)) :-> #t(:sort var-decl)))
       (#^dom
        . #@("pvs" := (#t(:list (:sort dep-type-expr))) :->
             #t(:sort dep-type-exprs)))
       (#^adformals
        . #@("pvs" := (#t(:list (:sort adformal))) :->
             #t(:sort adformals)))
       (#^⊂ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊢ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^<\| . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^∃ . #@("pvs" := nil :-> #t(:sort exists-op)))
       (#^fact . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^adtbody
        . #@("pvs" :=
             (#t(:list
                 (:union (:sort importing-elt) (:sort assuming-part)
                  (:sort id-datatype-elt))))
             :-> #t(:sort adtbody)))
       (#^function . #@("pvs" := nil :-> #t(:sort fun-array)))
       (#^nosemic . #@("pvs" := nil :-> #t(:sort assuming-opt-0)))
       (#^fun-arguments
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort fun-arguments)))
       (#^⊣ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊃ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^unary-term-expr
        . #@("pvs" := (#t(:sort ◯) #t(:sort expr)) :-> #t(:sort expr)))
       (#^& . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^dep-binding
        . #@("pvs" := (#t(:sort idop) #t(:sort type-expr)) :->
             #t(:sort dep-type-expr)))
       (#^assignment
        . #@("pvs" :=
             (#t(:sort args) #t(:sort expr) #t(:sort assignment-alt-0))
             :-> #t(:sort assignment)))
       (#^≙ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^varianttype
        . #@("pvs" := (#t(:sort fields)) :-> #t(:sort recordtype)))
       (#^assign-id
        . #@("pvs" := (#t(:sort sb-runtime::id)) :->
             #t(:sort assign-arg)))
       (#^<< . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊄ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^else . #@("pvs" := nil :-> #t(:sort table-entry-opt-0)))
       (#^exporting-all
        . #@("pvs" := (#t(:sort exportings-opt-0)) :->
             #t(:sort exportings)))
       (#^≺ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^□ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^iff . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^§ . #@("pvs" := nil :-> #t(:sort semic)))
       (#^ids
        . #@("pvs" := (#t(:list (:sort sb-runtime::id))) :->
             #t(:sort ids)))
       (#^idops
        . #@("pvs" := (#t(:list (:sort idop))) :-> #t(:sort idops)))
       (#^↦ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^adtcase-subtype
        . #@("pvs" :=
             (#t(:sort adtcase-opt-0) #t(:sort pidop) #t(:sort pidop))
             :-> #t(:sort adtcase)))
       (#^theory-formals
        . #@("pvs" := (#t(:list (:sort theory-formal))) :->
             #t(:sort theory-formals)))
       (#^⊥ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊅ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noeq . #@("pvs" := nil :-> #t(:sort lib-decl-opt-0)))
       (#^nolib . #@("pvs" := nil :-> #t(:sort modname-opt-0)))
       (#^has-type . #@("pvs" := nil :-> #t(:sort judgement-opt-0)))
       (#^bind
        . #@("pvs" := (#t(:sort bind-alt-0) #t(:sort expr)) :->
             #t(:sort bind)))
       (#^✠ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^no-pred . #@("pvs" := nil :-> #t(:sort typed-id-opt-0)))
       (#^≻ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^rec-expr
        . #@("pvs" := (#t(:sort assignments)) :-> #t(:sort expr)))
       (#^selection
        . #@("pvs" :=
             (#t(:sort pidop) #t(:sort selection-opt-0) #t(:sort expr))
             :-> #t(:sort selection)))
       (#^id-datatype-elt-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort id-datatype-elt-opt-0)))
       (#^/= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊆ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^expmodclosure . #@("pvs" := nil :-> #t(:sort exportingmods)))
       (#^~ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^^ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^∇ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^∧ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^mapping-subst
        . #@("pvs" := (#t(:sort mapping-lhs) #t(:sort mapping-rhs)) :->
             #t(:sort theory-decl-mapping)))
       (#^noatdecl . #@("pvs" := nil :-> #t(:sort assuming-opt-3)))
       (#^ftype-decl
        . #@("pvs" :=
             (#t(:sort type-keyword)
              #t(:sort theory-formal-decl-opt-0))
             :-> #t(:sort theory-formal-decl)))
       (#^if-expr
        . #@("pvs" :=
             (#t(:sort expr) #t(:sort expr) #t(:sort elsifs)
              #t(:sort expr))
             :-> #t(:sort expr)))
       (#^⊇ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^* . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^exporting
        . #@("pvs" := (#t(:sort exportings) #t(:sort exporting-opt-0))
             :-> #t(:sort exporting)))
       (#^fields
        . #@("pvs" := (#t(:list (:sort field-decls))) :->
             #t(:sort fields)))
       (#^∈ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^∨ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^theory . #@("pvs" := nil :-> #t(:sort mapping-lhs-opt-0)))
       (#^\\/ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^term-expr
        . #@("pvs" := (#t(:sort √) #t(:sort expr-args)) :->
             #t(:sort expr)))
       (#^jdecls
        . #@("pvs" := (#t(:list (:sort jdecl))) :-> #t(:sort jdecls)))
       (#^claim . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^⊨ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noqual . #@("pvs" := nil :-> #t(:sort mapping-lhs-opt-0)))
       (#^+ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^∩ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^∉ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^exptype . #@("pvs" := nil :-> #t(:sort expname-opt-0)))
       (#^set-list-expr
        . #@("pvs" := (#t(:list (:sort expr))) :-> #t(:sort set-expr)))
       (#^⌜⌝ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^field-decls
        . #@("pvs" := (#t(:sort ids) #t(:sort type-expr)) :->
             #t(:sort field-decls)))
       (#^formal-importing-elt
        . #@("pvs" :=
             (#t(:sort theory-formal-opt-1) #t(:sort importing)) :->
             #t(:sort theory-formal-opt-0)))
       (#^id-theory-decl
        . #@("pvs" :=
             (#t(:sort pidop) #t(:sort theory-decl)
              #t(:sort id-datatype-elt-opt-0))
             :-> #t(:sort id-datatype-elt)))
       (#^⨀ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^eager
        . #@("pvs" := nil :-> #t(:sort rewrite-name-or-fnum-opt-0)))
       (#^fnum
        . #@("pvs" := (#t(:sort number) #t(:sort -)) :->
             #t(:sort fnum)))
       (#^auto-rewriteminus
        . #@("pvs" := (#t(:list (:sort rewrite-name))) :->
             #t(:sort auto-rewrite)))
       (#^⊩ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^pidop
        . #@("pvs" := (#t(:list (:sort idop))) :-> #t(:sort pidop)))
       (#^noelse . #@("pvs" := nil :-> #t(:sort expr-opt-2)))
       (#^★ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^subtype-of . #@("pvs" := nil :-> #t(:sort judgement-opt-0)))
       (#^ifappl . #@("pvs" := (#t(:sort expr)) :-> #t(:sort expr)))
       (#^∪ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^nocontains . #@("pvs" := nil :-> #t(:sort type-decl-opt-0)))
       (#^¬ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^law . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^⨁ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^⊊ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^- . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^simplebind
        . #@("pvs" :=
             (#t(:sort idop) #t(:sort pdf) #t(:sort ename-opt-0)) :->
             #t(:sort simplebind)))
       (#^∋ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^arguments
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort arguments)))
       (#^++ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^lemma . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^⨂ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^theory-abbreviation-decl
        . #@("pvs" := (#t(:sort modname) #t(:sort pidop)) :->
             #t(:sort importing-item)))
       (#^⊋ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noexpbuts . #@("pvs" := nil :-> #t(:sort exportings-opt-0)))
       (#^«» . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^semic
        . #@("pvs" := (#t(:sort semic)) :-> #t(:sort assuming-opt-0)))
       (#^expr-args
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort expr-args)))
       (#^** . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^auto-rewrite
        . #@("pvs" := (#t(:list (:sort rewrite-name))) :->
             #t(:sort auto-rewrite)))
       (#^formula . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^macro-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort expr)) :->
             #t(:sort macro-decl)))
       (#^notheory . #@("pvs" := nil :-> #t(:sort theory-opt-2)))
       (#^name-expr
        . #@("pvs" :=
             (#t(:sort name) #t(:sort ename-opt-0)
              #t(:sort name-expr-opt-0))
             :-> #t(:sort name-expr)))
       (#^colrest
        . #@("pvs" := (#t(:list (:union (:sort expr) (:op #^else))))
             :-> #t(:sort colrest)))
       (#^lazy
        . #@("pvs" := nil :-> #t(:sort rewrite-name-or-fnum-opt-0)))
       (#^orelse . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^type-appl
        . #@("pvs" := (#t(:sort name) #t(:sort arguments)) :->
             #t(:sort type-ex)))
       (#^/ . #@("pvs" := nil :-> #t(:sort opsym))))))


(defparameter *PVS-SORT-TABLE*
    (make-sort-table
     '((#t(:sort set-formals)
        . #t(:union (:op #^set-formals) (:sort set-formal)))
       (#t(:sort table-entry-opt-0)
        . #t(:union (:op #^else) (:sort expr) (:op #^notableentry)))
       (#t(:sort datatype)
        . #t(:union (:op #^codatatypes) (:op #^codatatype)
             (:op #^datatypes) (:op #^datatype)))
       (#t(:sort table-expr-opt-0)
        . #t(:union (:sort expr) (:op #^norowvar)))
       (#t(:sort assuming-opt-1)
        . #t(:union (:op #^sec) (:op #^no-sec)))
       (#t(:sort exportings)
        . #t(:union (:op #^expnames) (:op #^exporting-all)))
       (#t(:sort idop) . #t(:op #^idop))
       (#t(:sort expr-opt-1)
        . #t(:union (:sort expr) (:op #^expr-NULL-1)))
       (#t(:sort theory-opt-2)
        . #t(:union (:sort theory-part) (:op #^notheory)))
       (#t(:sort rewrite-name-or-fnum-opt-0)
        . #t(:union (:op #^macro) (:op #^eager) (:op #^lazy)))
       (#t(:sort adt-or-theories) . #t(:op #^adt-or-modules))
       (#t(:sort ⊎) . #t(:op #^⊎))
       (#t(:sort implies) . #t(:op #^implies))
       (#t(:sort table-expr) . #t(:op #^table-expr))
       (#t(:sort ±) . #t(:op #^±))
       (#t(:sort theory-declaration)
        . #t(:union (:sort judgement) (:sort datatype)
             (:sort formula-decl) (:sort coind-decl) (:sort corec-decl)
             (:sort ind-decl) (:sort def-decl) (:sort macro-decl)
             (:sort const-decl) (:sort var-decl) (:sort units-decl)
             (:sort type-decl) (:sort theory-decl) (:sort lib-decl)))
       (#t(:sort type-ex)
        . #t(:union (:op #^extended-type) (:sort type-expr-sans-name)
             (:op #^extended-type-appl) (:op #^extended-type-name)
             (:op #^type-appl) (:op #^type-name)))
       (#t(:sort ←) . #t(:op #^←))
       (#t(:sort theory-decl-modname) . #t(:op #^modname))
       (#t(:sort assuming-opt-3)
        . #t(:union (:op #^atdecl) (:op #^noatdecl)))
       (#t(:sort ⊏) . #t(:op #^⊏)) (#t(:sort >>=) . #t(:op #^>>=))
       (#t(:sort table-entry) . #t(:op #^table-entry))
       (#t(:sort theory-formal-decl)
        . #t(:union (:op #^ftheory-decl) (:op #^fconst-decl)
             (:op #^ftype-decl)))
       (#t(:sort //) . #t(:op #^//))
       (#t(:sort bracket-expr)
        . #t(:union (:op #^bracket-expr) (:op #^list-expr)
             (:sort tuple-expr)))
       (#t(:sort formula-name)
        . #t(:union (:op #^theorem) (:op #^sublemma)
             (:op #^proposition) (:op #^postulate) (:op #^obligation)
             (:op #^lemma) (:op #^law) (:op #^formula) (:op #^fact)
             (:op #^corollary) (:op #^conjecture) (:op #^claim)
             (:op #^challenge) (:op #^axiom)))
       (#t(:sort ↑) . #t(:op #^↑)) (#t(:sort ⊐) . #t(:op #^⊐))
       (#t(:sort typed-ids) . #t(:op #^typed-ids))
       (#t(:sort comp-type-expr)
        . #t(:union (:op #^funtype) (:sort recordtype)))
       (#t(:sort ⌊⌋) . #t(:op #^⌊⌋))
       (#t(:sort importing-item)
        . #t(:union (:op #^theory-abbreviation-decl) (:sort modname)))
       (#t(:sort →) . #t(:op #^→))
       (#t(:sort forall-op) . #t(:union (:op #^∀) (:op #^forall)))
       (#t(:sort selection-opt-0)
        . #t(:union (:sort typed-ids) (:op #^selection-NULL-1)))
       (#t(:sort ⊑) . #t(:op #^⊑)) (#t(:sort xor) . #t(:op #^xor))
       (#t(:sort bind-alt-0)
        . #t(:union (:op #^binds) (:sort simplebind)))
       (#t(:sort <>) . #t(:op #^<>)) (#t(:sort and) . #t(:op #^and))
       (#t(:sort |##|) . #t(:op #^|##|))
       (#t(:sort assumption) . #t(:op #^assumption))
       (#t(:sort theory-decl-mapping)
        . #t(:union (:op #^mapping-rename) (:op #^mapping-subst)))
       (#t(:sort ↓) . #t(:op #^↓))
       (#t(:sort lambda-body) . #t(:op #^lambda-body))
       (#t(:sort ⊒) . #t(:op #^⊒))
       (#t(:sort expr)
        . #t(:union (:sort table-expr) (:op #^cond-expr)
             (:op #^cases-expr) (:op #^update-expr) (:op #^where-expr)
             (:op #^let-expr) (:sort set-expr) (:op #^name-bind-expr)
             (:op #^skovar) (:op #^bind-expr) (:op #^if-expr)
             (:op #^ifappl) (:op #^coercion) (:op #^unary-term-expr)
             (:op #^term-expr) (:op #^application) (:op #^projappl)
             (:op #^fieldappl) (:op #^projex) (:op #^fieldex)
             (:op #^rec-expr) (:sort bracket-expr) (:sort name-expr)
             (:op #^string-expr)))
       (#t(:sort expname) . #t(:op #^expname))
       (#t(:sort =>) . #t(:op #^=>))
       (#t(:sort rewrite-name) . #t(:op #^rewrite-name))
       (#t(:sort expr-alt-0)
        . #t(:union (:sort exists-op) (:sort forall-op)
             (:sort lambda-op)))
       (#t(:sort ∓) . #t(:op #^∓))
       (#t(:sort typed-id) . #t(:op #^typed-id))
       (#t(:sort units-decl) . #t(:op #^units-decl))
       (#t(:sort <=) . #t(:op #^<=)) (#t(:sort ⊓) . #t(:op #^⊓))
       (#t(:sort >>) . #t(:op #^>>))
       (#t(:sort def-decl) . #t(:op #^def-decl))
       (#t(:sort const-decl)
        . #t(:union (:op #^const-decl) (:op #^uninterp-const-decl)))
       (#t(:sort ∔) . #t(:op #^∔))
       (#t(:sort cond-case) . #t(:op #^cond-case))
       (#t(:sort theory-formal-opt-0)
        . #t(:union (:op #^formal-importing-elt)
             (:op #^theory-formal-NULL-1)))
       (#t(:sort ==) . #t(:op #^==)) (#t(:sort ⊔) . #t(:op #^⊔))
       (#t(:sort dep-type-expr)
        . #t(:union (:op #^dep-binding) (:sort type-expr)))
       (#t(:sort mapping-lhs-opt-0)
        . #t(:union (:op #^typed) (:op #^theory) (:op #^type)
             (:op #^noqual)))
       (#t(:sort ≠) . #t(:op #^≠)) (#t(:sort >=) . #t(:op #^>=))
       (#t(:sort ⊕) . #t(:op #^⊕)) (#t(:sort or) . #t(:op #^or))
       (#t(:sort typed-id-opt-0)
        . #t(:union (:sort expr) (:op #^no-pred)))
       (#t(:sort lambda-op) . #t(:union (:op #^λ) (:op #^lambda)))
       (#t(:sort idop-not-number) . #t(:op #^idop))
       (#t(:sort ⋀) . #t(:op #^⋀)) (#t(:sort btup) . #t(:op #^btup))
       (#t(:sort expname-opt-0)
        . #t(:union (:op #^expformula) (:op #^exptype)
             (:sort type-expr) (:op #^noexpkind)))
       (#t(:sort ≡) . #t(:op #^≡)) (#t(:sort ≁) . #t(:op #^≁))
       (#t(:sort ⊖) . #t(:op #^⊖)) (#t(:sort names) . #t(:op #^names))
       (#t(:sort ⋁) . #t(:op #^⋁)) (#t(:sort ⊗) . #t(:op #^⊗))
       (#t(:sort ∘) . #t(:op #^∘))
       (#t(:sort theory-formal-opt-1)
        . #t(:union (:op #^lib-fdecl) (:op #^nolib)))
       (#t(:sort ⋂) . #t(:op #^⋂)) (#t(:sort pdf) . #t(:op #^pdf))
       (#t(:sort conversion)
        . #t(:union (:op #^conversionminus) (:op #^conversionplus)
             (:op #^conversion)))
       (#t(:sort ≃) . #t(:op #^≃)) (#t(:sort ⌞⌟) . #t(:op #^⌞⌟))
       (#t(:sort assignment-alt-0)
        . #t(:union (:op #^arr) (:op #^ceq)))
       (#t(:sort ⊘) . #t(:op #^⊘))
       (#t(:sort mapping) . #t(:op #^mapping-subst))
       (#t(:sort ⋃) . #t(:op #^⋃))
       (#t(:sort assignments) . #t(:op #^assignments))
       (#t(:sort assuming)
        . #t(:union (:op #^assuming) (:op #^auto-rewrite-elt)
             (:op #^conversion-elt) (:op #^judgement-elt)
             (:op #^importing-elt)))
       (#t(:sort ⊙) . #t(:op #^⊙)) (#t(:sort ≤) . #t(:op #^≤))
       (#t(:sort expr-opt-0)
        . #t(:union (:sort actuals) (:op #^noactuals)))
       (#t(:sort <) . #t(:op #^<)) (#t(:sort √) . #t(:op #^√))
       (#t(:sort mapping-rhs)
        . #t(:union (:sort comp-type-expr) (:sort expr)))
       (#t(:sort <<=) . #t(:op #^<<=))
       (#t(:sort idopappl) . #t(:union (:op #^idopappl) (:sort idop)))
       (#t(:sort assuming-opt-0)
        . #t(:union (:op #^semic) (:op #^nosemic)))
       (#t(:sort lib-decl-opt-0) . #t(:union (:op #^eq) (:op #^noeq)))
       (#t(:sort =) . #t(:op #^=)) (#t(:sort ≥) . #t(:op #^≥))
       (#t(:sort table-expr-opt-1)
        . #t(:union (:sort expr) (:op #^nocolvar)))
       (#t(:sort expr-opt-2) . #t(:union (:sort expr) (:op #^noelse)))
       (#t(:sort ◁) . #t(:op #^◁)) (#t(:sort ⌈⌉) . #t(:op #^⌈⌉))
       (#t(:sort ▷) . #t(:op #^▷)) (#t(:sort >) . #t(:op #^>))
       (#t(:sort ≦) . #t(:op #^≦))
       (#t(:sort set-expr)
        . #t(:union (:op #^set-list-expr) (:op #^set-expr)))
       (#t(:sort ⊛) . #t(:op #^⊛))
       (#t(:sort exportings-opt-0)
        . #t(:union (:op #^expbuts) (:op #^noexpbuts)))
       (#t(:sort ∼) . #t(:op #^∼)) (#t(:sort ↝) . #t(:op #^↝))
       (#t(:sort ≧) . #t(:op #^≧)) (#t(:sort ≇) . #t(:op #^≇))
       (#t(:sort elsif) . #t(:op #^elsif))
       (#t(:sort actuals)
        . #t(:union (:op #^actuals2) (:sort actuals1)))
       (#t(:sort ≨) . #t(:op #^≨)) (#t(:sort ≈) . #t(:op #^≈))
       (#t(:sort type-coercion) . #t(:op #^type-coercion))
       (#t(:sort modname) . #t(:op #^modname))
       (#t(:sort dep-type-exprs)
        . #t(:union (:op #^cotupletype) (:op #^dom)))
       (#t(:sort ⋈) . #t(:op #^⋈))
       (#t(:sort type-decl)
        . #t(:union (:op #^type-decl) (:op #^uninterp-type-decl)))
       (#t(:sort ⊞) . #t(:op #^⊞)) (#t(:sort ≩) . #t(:op #^≩))
       (#t(:sort ≉) . #t(:op #^≉))
       (#t(:sort ename-opt-0)
        . #t(:union (:sort type-expr) (:op #^notype)))
       (#t(:sort elsifs) . #t(:op #^elsifs))
       (#t(:sort type-keyword)
        . #t(:union (:op #^typeplus) (:op #^nonempty-type)
             (:op #^type)))
       (#t(:sort mappings) . #t(:op #^mappings))
       (#t(:sort not) . #t(:op #^not))
       (#t(:sort opsym)
        . #t(:union (:op #^⌞⌟) (:op #^⌜⌝) (:op #^⌊⌋) (:op #^⌈⌉)
             (:op #^⟪⟫) (:op #^⟦⟧) (:op #^〈〉) (:op #^«») (:op #^✠)
             (:op #^★) (:op #^◯) (:op #^⊩) (:op #^⊥) (:op #^⊣)
             (:op #^∇) (:op #^⇔) (:op #^⇓) (:op #^⇑) (:op #^⇐)
             (:op #^↦) (:op #^↝) (:op #^↓) (:op #^→) (:op #^↑)
             (:op #^←) (:op #^•) (:op #^⋃) (:op #^⋂) (:op #^⋁)
             (:op #^⋀) (:op #^⊔) (:op #^⊓) (:op #^⊒) (:op #^⊑)
             (:op #^⊐) (:op #^⊏) (:op #^⊎) (:op #^⊋) (:op #^⊊)
             (:op #^⊇) (:op #^⊆) (:op #^⊅) (:op #^⊄) (:op #^⊃)
             (:op #^⊂) (:op #^∪) (:op #^∩) (:op #^∋) (:op #^∉)
             (:op #^∈) (:op #^▷) (:op #^◁) (:op #^≻) (:op #^≺)
             (:op #^≱) (:op #^≰) (:op #^≯) (:op #^≮) (:op #^≫)
             (:op #^≪) (:op #^≩) (:op #^≨) (:op #^≧) (:op #^≦)
             (:op #^≥) (:op #^≤) (:op #^⋈) (:op #^≡) (:op #^≙)
             (:op #^≗) (:op #^≐) (:op #^≏) (:op #^≎) (:op #^≍)
             (:op #^≉) (:op #^∼) (:op #^≈) (:op #^≇) (:op #^≅)
             (:op #^≃) (:op #^≁) (:op #^⊠) (:op #^⊟) (:op #^⊞)
             (:op #^÷) (:op #^×) (:op #^∔) (:op #^∓) (:op #^±)
             (:op #^⊨) (:op #^⊢) (:op #^⨀) (:op #^⨂) (:op #^⨁)
             (:op #^⊙) (:op #^⊘) (:op #^⊗) (:op #^⊖) (:op #^⊕)
             (:op #^∘) (:op #^{\|\|}) (:op #^|(\|\|)|) (:op #^[\|\|])
             (:op #^\|>) (:op #^<\|) (:op #^\|=) (:op #^\|-)
             (:op #^|##|) (:op #^@@) (:op #^|#|) (:op #^>>=)
             (:op #^<<=) (:op #^>>) (:op #^<<) (:op #^^^) (:op #^//)
             (:op #^**) (:op #^++) (:op #^andthen) (:op #^orelse)
             (:op #^xor) (:op #^o) (:op #^false) (:op #^true)
             (:op #^if) (:op #^<=>) (:op #^=>) (:op #^~) (:op #^<>)
             (:op #^◇) (:op #^□) (:op #^^) (:op #^\\/) (:op #^/\\)
             (:op #^∨) (:op #^∧) (:op #^⇒) (:op #^&) (:op #^¬)
             (:op #^not) (:op #^and) (:op #^or) (:op #^when)
             (:op #^implies) (:op #^iff) (:op #^>=) (:op #^>)
             (:op #^<=) (:op #^<) (:op #^==) (:op #^/=) (:op #^≠)
             (:op #^=) (:op #^/) (:op #^*) (:op #^-) (:op #^+)))
       (#t(:sort name-expr-opt-0)
        . #t(:union (:sort expr) (:op #^nopred)))
       (#t(:sort ≪) . #t(:op #^≪)) (#t(:sort ⊟) . #t(:op #^⊟))
       (#t(:sort <=>) . #t(:op #^<=>))
       (#t(:sort args) . #t(:op #^args)) (#t(:sort @@) . #t(:op #^@@))
       (#t(:sort /\\) . #t(:op #^/\\)) (#t(:sort ≫) . #t(:op #^≫))
       (#t(:sort ◇) . #t(:op #^◇))
       (#t(:sort assumings) . #t(:op #^assuming-part))
       (#t(:sort datatype-alt-0)
        . #t(:union (:sort id-datatype-elt) (:sort assuming-part)
             (:sort importing-elt)))
       (#t(:sort lib-decl) . #t(:op #^lib-decl))
       (#t(:sort theory-decl-modname-opt-1)
        . #t(:union (:sort theory-decl-modname) (:op #^notgt)))
       (#t(:sort def-decl-opt-0)
        . #t(:union (:sort expr) (:op #^noexpr)))
       (#t(:sort formula-decl) . #t(:op #^formula-decl))
       (#t(:sort mapping-lhs) . #t(:op #^mapping-lhs))
       (#t(:sort jdecl)
        . #t(:union (:op #^jdecl-type) (:op #^jdecl-expr)))
       (#t(:sort o) . #t(:op #^o))
       (#t(:sort adtdecl) . #t(:op #^adtdecl))
       (#t(:sort let-bindings) . #t(:op #^let-bindings))
       (#t(:sort ≍) . #t(:op #^≍)) (#t(:sort ename) . #t(:op #^ename))
       (#t(:sort bname) . #t(:union (:op #^bname) (:sort name)))
       (#t(:sort importing-elt) . #t(:op #^importing-elt))
       (#t(:sort col-heading) . #t(:op #^col-heading))
       (#t(:sort exporting-opt-0)
        . #t(:union (:sort exportingmods) (:op #^noexportingmods)))
       (#t(:sort comp-type-expr-opt-1)
        . #t(:union (:sort dep-type-exprs) (:op #^emptytuple)))
       (#t(:sort coind-decl) . #t(:op #^coind-decl))
       (#t(:sort ≮) . #t(:op #^≮)) (#t(:sort ≎) . #t(:op #^≎))
       (#t(:sort pidops) . #t(:op #^pidops))
       (#t(:sort rational) . #t(:op #^rational))
       (#t(:sort adtcase)
        . #t(:union (:op #^adtcase-subtype) (:op #^adtcase)))
       (#t(:sort units-expr)
        . #t(:union (:op #^units-term) (:sort name)))
       (#t(:sort modnames) . #t(:op #^modnames))
       (#t(:sort ≯) . #t(:op #^≯)) (#t(:sort ≏) . #t(:op #^≏))
       (#t(:sort ⇐) . #t(:op #^⇐))
       (#t(:sort idopappls) . #t(:op #^idopappls))
       (#t(:sort importing) . #t(:op #^importing))
       (#t(:sort ⟦⟧) . #t(:op #^⟦⟧)) (#t(:sort \|-) . #t(:op #^\|-))
       (#t(:sort theory-decl-mappings) . #t(:op #^mappings))
       (#t(:sort ≰) . #t(:op #^≰)) (#t(:sort ≐) . #t(:op #^≐))
       (#t(:sort modname-opt-0)
        . #t(:union (:sort sb-runtime::id) (:op #^nolib)))
       (#t(:sort andthen) . #t(:op #^andthen))
       (#t(:sort type-def) . #t(:op #^type-def))
       (#t(:sort ⇑) . #t(:op #^⇑))
       (#t(:sort adtcase-opt-0)
        . #t(:union (:sort constructor) (:op #^adtcase-NULL-1)))
       (#t(:sort rewrite-name-or-fnum)
        . #t(:union (:sort rewrite-name) (:op #^rewrite-fnum)))
       (#t(:sort lambda-formals)
        . #t(:union (:op #^lambda-formals) (:sort lambda-formal)))
       (#t(:sort constructor) . #t(:op #^adtdecls))
       (#t(:sort lambda-formal)
        . #t(:union (:sort pdformals) (:sort idop-not-number)))
       (#t(:sort table-entries) . #t(:op #^table-entries))
       (#t(:sort |(\|\|)|) . #t(:op #^|(\|\|)|))
       (#t(:sort ≱) . #t(:op #^≱))
       (#t(:sort col-heading-alt-0)
        . #t(:union (:op #^else) (:sort expr)))
       (#t(:sort tuple-expr) . #t(:op #^tuple-expr))
       (#t(:sort adt-or-theory) . #t(:op #^adt-or-module))
       (#t(:sort ⇒) . #t(:op #^⇒))
       (#t(:sort modname-opt-2)
        . #t(:union (:sort modname) (:op #^notgt)))
       (#t(:sort theory-decl) . #t(:op #^theory-decl))
       (#t(:sort \|>) . #t(:op #^\|>))
       (#t(:sort corec-decl) . #t(:op #^corec-decl))
       (#t(:sort ^^) . #t(:op #^^^)) (#t(:sort ⇓) . #t(:op #^⇓))
       (#t(:sort assign-arg)
        . #t(:union (:op #^assign-num) (:op #^assign-skoname)
             (:op #^assign-id) (:op #^proj-assign) (:op #^id-assign)
             (:op #^assign-tuple)))
       (#t(:sort judgement) . #t(:op #^judgement))
       (#t(:sort theory-part) . #t(:op #^theory-part))
       (#t(:sort set-formal)
        . #t(:union (:sort pdformals) (:op #^set-id)))
       (#t(:sort name) . #t(:op #^name))
       (#t(:sort actual)
        . #t(:union (:sort mappings) (:sort comp-type-expr)
             (:sort expr)))
       (#t(:sort ◯) . #t(:op #^◯))
       (#t(:sort unique-name) . #t(:op #^unique-name))
       (#t(:sort ⇔) . #t(:op #^⇔)) (#t(:sort \|=) . #t(:op #^\|=))
       (#t(:sort assuming-part) . #t(:sort assumings))
       (#t(:sort when) . #t(:op #^when)) (#t(:sort •) . #t(:op #^•))
       (#t(:sort ind-decl) . #t(:op #^ind-decl))
       (#t(:sort ⟪⟫) . #t(:op #^⟪⟫))
       (#t(:sort theory-formal) . #t(:op #^theory-formal))
       (#t(:sort ÷) . #t(:op #^÷)) (#t(:sort ×) . #t(:op #^×))
       (#t(:sort recordtype)
        . #t(:union (:op #^varianttype) (:op #^recordtype)))
       (#t(:sort judgement-opt-0)
        . #t(:union (:op #^rec-subtype-of) (:op #^rec-has-type)
             (:op #^subtype-of) (:op #^has-type)))
       (#t(:sort ⊠) . #t(:op #^⊠)) (#t(:sort |#|) . #t(:op #^|#|))
       (#t(:sort 〈〉) . #t(:op #^〈〉))
       (#t(:sort {\|\|}) . #t(:op #^{\|\|}))
       (#t(:sort [\|\|]) . #t(:op #^[\|\|]))
       (#t(:sort theory-decl-modname-opt-0)
        . #t(:union (:sort theory-decl-mappings) (:op #^nomap)))
       (#t(:sort theory-formal-decl-opt-0)
        . #t(:union (:op #^struct-subtype) (:sort type-expr)
             (:op #^notype)))
       (#t(:sort table-expr-opt-2)
        . #t(:union (:sort col-heading) (:op #^nocolheading)))
       (#t(:sort ≗) . #t(:op #^≗))
       (#t(:sort deptypes) . #t(:op #^deptypes))
       (#t(:sort var-decl) . #t(:op #^var-decl))
       (#t(:sort type-expr) . #t(:union (:op #^ghost) (:sort type-ex)))
       (#t(:sort adformals) . #t(:op #^adformals))
       (#t(:sort modname-opt-1)
        . #t(:union (:sort mappings) (:op #^nomap)))
       (#t(:sort exportingmods)
        . #t(:union (:sort modnames) (:op #^expmodclosure)
             (:op #^expmodall)))
       (#t(:sort ⊂) . #t(:op #^⊂)) (#t(:sort ⊢) . #t(:op #^⊢))
       (#t(:sort <\|) . #t(:op #^<\|))
       (#t(:sort lambda-formals-opt-0)
        . #t(:union (:op #^comma) (:op #^nocomma)))
       (#t(:sort adtbody) . #t(:op #^adtbody))
       (#t(:sort fun-arguments) . #t(:op #^fun-arguments))
       (#t(:sort ⊣) . #t(:op #^⊣)) (#t(:sort ⊃) . #t(:op #^⊃))
       (#t(:sort adt-or-theory-opt-0)
        . #t(:union (:sort theory-formals) (:op #^no-theory-formals)))
       (#t(:sort &) . #t(:op #^&))
       (#t(:sort actuals1) . #t(:op #^actuals))
       (#t(:sort assignment) . #t(:op #^assignment))
       (#t(:sort assuming-opt-2)
        . #t(:union (:op #^pdf) (:op #^noformals)))
       (#t(:sort ≙) . #t(:op #^≙))
       (#t(:sort type-expr-sans-name)
        . #t(:union (:sort comp-type-expr) (:op #^subtype)
             (:op #^enumtype) (:op #^expr-as-type)))
       (#t(:sort assuming-decl)
        . #t(:union (:sort assumption) (:sort theory-declaration)))
       (#t(:sort <<) . #t(:op #^<<)) (#t(:sort ⊄) . #t(:op #^⊄))
       (#t(:sort adformal)
        . #t(:union (:sort typed-ids) (:sort typed-id)))
       (#t(:sort pdformals) . #t(:sort adformals))
       (#t(:sort id-datatype-elt-opt-0)
        . #t(:union (:sort semic) (:op #^id-datatype-elt-NULL-1)))
       (#t(:sort ≺) . #t(:op #^≺)) (#t(:sort □) . #t(:op #^□))
       (#t(:sort iff) . #t(:op #^iff))
       (#t(:sort unique-name-opt-0)
        . #t(:union (:op #^formula) (:op #^typed) (:op #^type)
             (:op #^noqual)))
       (#t(:sort ids) . #t(:op #^ids))
       (#t(:sort idops) . #t(:op #^idops)) (#t(:sort ↦) . #t(:op #^↦))
       (#t(:sort theory-formals) . #t(:op #^theory-formals))
       (#t(:sort theory-elt)
        . #t(:union (:op #^theory) (:op #^auto-rewrite-elt)
             (:op #^conversion-elt) (:op #^judgement-elt)
             (:op #^importing-elt)))
       (#t(:sort theory-opt-1)
        . #t(:union (:sort assuming-part) (:op #^noass)))
       (#t(:sort ⊥) . #t(:op #^⊥)) (#t(:sort ⊅) . #t(:op #^⊅))
       (#t(:sort bind) . #t(:op #^bind)) (#t(:sort ✠) . #t(:op #^✠))
       (#t(:sort ≻) . #t(:op #^≻)) (#t(:sort /=) . #t(:op #^/=))
       (#t(:sort ⊆) . #t(:op #^⊆))
       (#t(:sort selection) . #t(:op #^selection))
       (#t(:sort ~) . #t(:op #^~)) (#t(:sort ^) . #t(:op #^^))
       (#t(:sort ∇) . #t(:op #^∇)) (#t(:sort ∧) . #t(:op #^∧))
       (#t(:sort *) . #t(:op #^*)) (#t(:sort ⊇) . #t(:op #^⊇))
       (#t(:sort fun-array)
        . #t(:union (:op #^array) (:op #^function)))
       (#t(:sort exporting) . #t(:op #^exporting))
       (#t(:sort fields) . #t(:op #^fields))
       (#t(:sort theory-opt-0)
        . #t(:union (:sort exporting) (:op #^noexp)))
       (#t(:sort ∈) . #t(:op #^∈)) (#t(:sort ∨) . #t(:op #^∨))
       (#t(:sort units-decl-opt-0)
        . #t(:union (:sort rational) (:op #^no-offset)))
       (#t(:sort theory) . #t(:op #^module))
       (#t(:sort \\/) . #t(:op #^\\/))
       (#t(:sort jdecls) . #t(:op #^jdecls))
       (#t(:sort ⊨) . #t(:op #^⊨)) (#t(:sort +) . #t(:op #^+))
       (#t(:sort exists-op) . #t(:union (:op #^∃) (:op #^exists)))
       (#t(:sort type-def-alt-0)
        . #t(:union (:op #^|<:|) (:op #^from) (:op #^equal)))
       (#t(:sort ∩) . #t(:op #^∩)) (#t(:sort ∉) . #t(:op #^∉))
       (#t(:sort rational-opt-1)
        . #t(:union (:sort number) (:op #^noden)))
       (#t(:sort ⌜⌝) . #t(:op #^⌜⌝))
       (#t(:sort field-decls) . #t(:op #^field-decls))
       (#t(:sort ⨀) . #t(:op #^⨀)) (#t(:sort fnum) . #t(:op #^fnum))
       (#t(:sort id-datatype-elt)
        . #t(:union (:op #^id-adtcase) (:op #^id-theory-decl)))
       (#t(:sort ⊩) . #t(:op #^⊩)) (#t(:sort pidop) . #t(:op #^pidop))
       (#t(:sort ★) . #t(:op #^★))
       (#t(:sort comp-type-expr-opt-0)
        . #t(:union (:sort fun-array) (:op #^comp-type-expr-NULL-1)))
       (#t(:sort ∪) . #t(:op #^∪)) (#t(:sort ¬) . #t(:op #^¬))
       (#t(:sort name-opt-0)
        . #t(:union (:op #^pidacts) (:sort pidop) (:op #^nomod)))
       (#t(:sort rational-opt-0) . #t(:union (:op #^neg) (:op #^pos)))
       (#t(:sort adt-or-theory-alt-0)
        . #t(:union (:sort datatype) (:sort theory)))
       (#t(:sort ⨁) . #t(:op #^⨁))
       (#t(:sort set-expr-opt-0)
        . #t(:union (:sort type-expr) (:op #^no-type-expr)))
       (#t(:sort ⊊) . #t(:op #^⊊)) (#t(:sort -) . #t(:op #^-))
       (#t(:sort simplebind) . #t(:op #^simplebind))
       (#t(:sort ∋) . #t(:op #^∋))
       (#t(:sort arguments) . #t(:op #^arguments))
       (#t(:sort ++) . #t(:op #^++))
       (#t(:sort const-value) . #t(:sort expr))
       (#t(:sort ⨂) . #t(:op #^⨂))
       (#t(:sort comp-type-expr-opt-2)
        . #t(:union (:sort type-expr) (:op #^comp-type-expr-NULL-2)))
       (#t(:sort ⊋) . #t(:op #^⊋))
       (#t(:sort semic) . #t(:union (:op #^§) (:op #^|;|)))
       (#t(:sort rewrite-name-opt-0)
        . #t(:union (:op #^formula) (:op #^type) (:op #^noqual)))
       (#t(:sort «») . #t(:op #^«»)) (#t(:sort **) . #t(:op #^**))
       (#t(:sort expr-args) . #t(:op #^expr-args))
       (#t(:sort auto-rewrite)
        . #t(:union (:op #^auto-rewriteminus) (:op #^auto-rewriteplus)
             (:op #^auto-rewrite)))
       (#t(:sort macro-decl) . #t(:op #^macro-decl))
       (#t(:sort name-expr) . #t(:op #^name-expr))
       (#t(:sort colrest) . #t(:op #^colrest))
       (#t(:sort orelse) . #t(:op #^orelse))
       (#t(:sort /) . #t(:op #^/))
       (#t(:sort type-decl-opt-0)
        . #t(:union (:sort expr) (:op #^nocontains))))))

(lang:lang-define 
:name "pvs"
:conc-name "pvs"
:code-package "PVS"
:abs-syn-package "PVS"
:use-packages '("NEWATTR" "LANG" "SB-RUNTIME" "SORT" "TERM" "OCC" "OPER" "ERGOLISP")
:sub-languages '("LEXICAL-TERMINALS")
:unparse-nts 't
:parse-routine-name 'PVS::PVS-PARSE
:unparse-routine-name 'PVS::PVS-UNPARSE
:win-unparse-routine-name 'PVS::PVS-WIN-UNPARSE
:sort-table-name 'PVS::*PVS-SORT-TABLE*
:opsig-table-name 'PVS::*PVS-OPSIG-TABLE*
)

